import java.util.*;
import java.awt.*;

public abstract class Bezier extends Polygon {
	protected int steps = 20;

	/**
	 * Linear interpolation between p[ia] and p[ib].
	 */
	protected void interpolation(Graphics g, Vector p, int ia, int ib) {
		int n = p.size();
		if (n < 2) return;
		if (ia < 0 || ia >= n) return;
		if (ib < 0 || ib >= n) return;
		Point point_a = (Point)p.elementAt(ia);
		Point point_b = (Point)p.elementAt(ib);
		g.drawLine(point_a.x, point_a.y, point_b.x, point_b.y);
	}

	/**
	 * Quadric interplation between (p[ia],p[ib]) and (p[ib],p[ic]).
	 */
	protected void interpolation(Graphics g, Vector p, int ia, int ib, int ic) {
		int n = p.size();
		if (n < 3) return;
		if (ia < 0 || ia >= n) return;
		if (ib < 0 || ib >= n) return;
		if (ic < 0 || ic >= n) return;
		
		double a, b, s = 1.0 / (double)steps;
		double x0, y0, x1, y1;
		double Ax, Ay, Bx, By, Cx, Cy;

		Ax = ((Point)p.elementAt(ia)).x;	Ay = ((Point)p.elementAt(ia)).y;
		Bx = ((Point)p.elementAt(ib)).x;	By = ((Point)p.elementAt(ib)).y;
		Cx = ((Point)p.elementAt(ic)).x;	Cy = ((Point)p.elementAt(ic)).y;
		a = 1.0; b = 1.0 - a;
		x0 = Ax*a*a + Bx*2.0*a*b + Cx*b*b;
		y0 = Ay*a*a + By*2.0*a*b + Cy*b*b;
		for (int j = 1; j <= steps; ++j) {
			a -= s; b = 1.0 - a;
			x1 = Ax*a*a + Bx*2.0*a*b + Cx*b*b;
			y1 = Ay*a*a + By*2.0*a*b + Cy*b*b;
			g.drawLine((int)x0, (int)y0, (int)x1, (int)y1);
			x0 = x1; y0 = y1;
		}
	}

	/**
	 * Cubic interplation between (p[ia],p[ib]) and (p[ic],p[id]).
	 */
	protected void interpolation(Graphics g, Vector p, int ia, int ib, int ic, int id) {
		int n = p.size();
		if (n < 4) return;
		if (ia < 0 || ia >= n) return;
		if (ib < 0 || ib >= n) return;
		if (ic < 0 || ic >= n) return;
		if (id < 0 || id >= n) return;
		
		double a, b, s = 1.0 / (double)steps;
		double x0, y0, x1, y1;
		double Ax, Ay, Bx, By, Cx, Cy, Dx, Dy;
		
		Ax = ((Point)p.elementAt(ia)).x;	Ay = ((Point)p.elementAt(ia)).y;
		Bx = ((Point)p.elementAt(ib)).x;	By = ((Point)p.elementAt(ib)).y;
		Cx = ((Point)p.elementAt(ic)).x;	Cy = ((Point)p.elementAt(ic)).y;
		Dx = ((Point)p.elementAt(id)).x;	Dy = ((Point)p.elementAt(id)).y;
		a = 1.0; b = 1.0 - a;
		x0 = Ax*a*a*a + Bx*3.0*a*a*b + Cx*3.0*a*b*b + Dx*b*b*b;
		y0 = Ay*a*a*a + By*3.0*a*a*b + Cy*3.0*a*b*b + Dy*b*b*b;
		for (int j = 1; j <= steps; ++j) {
			a -= s; b = 1.0 - a;
			x1 = Ax*a*a*a + Bx*3.0*a*a*b + Cx*3.0*a*b*b + Dx*b*b*b;
			y1 = Ay*a*a*a + By*3.0*a*a*b + Cy*3.0*a*b*b + Dy*b*b*b;
			g.drawLine((int)x0, (int)y0, (int)x1, (int)y1);
			x0 = x1; y0 = y1;
		}
	}
}

