import java.util.*;
import java.awt.*;

public class ClosedSpline extends Spline {
	public String toString() {
		return "Closed Spline";
	}

    public void compute(Vector p) {
		if (p.size() < 3) return;
		int n = p.size() + 1;
		x = new double[n];
		y = new double[n];
		for (int i = 0; i < p.size(); i++) {
			Point point = (Point)p.elementAt(i);
			x[i] = point.x;
			y[i] = point.y;
		}
		s = new double[n];
		xp = new double[n];
		yp = new double[n];
		x[x.length-1] = x[0];
		y[y.length-1] = y[0];
		s[0] = 0.0;
		for (int i = 1; i < n; i++) {
			s[i] = s[i-1] + Math.sqrt(sqr(x[i] - x[i-1]) + sqr(y[i] - y[i-1]));
		}
		derivatives(n, s, x, xp);
		derivatives(n, s, y, yp);
	}

    protected void derivatives(int n, double [] x, double [] y, double [] yp) {
		double h0 = x[1] - x[0];
		double h1 = x[2] - x[1];
		double hm3 = x[n-2] - x[n-3];
		double hm2 = x[n-1] - x[n-2];
		double d0 = (y[1] - y[0]) / h0 / h0;
		double d1 = (y[2] - y[1]) / h1 / h1;
		double dm3 = (y[n-2] - y[n-3]) / hm3 / hm3;
		double dm2 = (y[n-1] - y[n-2]) / hm2 / hm2;
		double hi = h0;
		double di = d0;

		double [] a = new double[n];
		double [] b = new double[n];
		double [] c = new double[n];
		double [] u = new double[n];
		double [] v = new double[n];

		for (int i = 0; i < n-1; i++) {
			double him1 = hi;
			hi = x[i+1] - x[i];
			double dim1 = di;
			di = (y[i+1] - y[i]) / hi / hi;
			a[i] = 2.0 / him1 + 2.0 / hi;
			b[i] = 1.0 / hi;
			c[i] = b[i];
			yp[i] = 3.0 * (di + dim1);
			u[i] = 0;
			v[i] = 3.0 * (di + dim1);
		}

		b[0] = 1.0 / h0;
		c[0] = b[0];
		a[0] = 2.0 * b[0] + 1.0 / hm2;
		a[n-2] = 2.0 / hm3 + 1.0 / hm2;
		u[0] = 1.0;
		u[n-2] = 1.0;
		v[0] = 3.0  * (d0 + dm2);
		solveTridia2(n-1, c, a, b, u, v);
		double fak = (v[0] + v[n-2]) / (u[0] + u[n-2] + hm2);
		for (int i = 0; i < n-1; i++) {
			yp[i] = v[i] - fak * u[i];
		}
		yp[n-1] = yp[0];
	}
}

