import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class PolygonCanvas extends Canvas {
    private Vector points = new Vector();
    private boolean showPoints;
    private int movingPointIndex = -1;
	private Polygon poly = null;

    public PolygonCanvas() {
		addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent event) {
				movingPointIndex = findIndexOf(event.getPoint(), 3.0);
			}

			public void mouseReleased(MouseEvent event) {
				movingPointIndex = -1;
			}

			public void mouseClicked(MouseEvent event) {
				addPoint(event.getPoint());
			}
		});
		
		addMouseMotionListener(new MouseMotionAdapter()	{
			public void mouseDragged(MouseEvent event) {
				onDraggedTo(event.getPoint());
			}
		});
	}

	private void onDraggedTo(Point dest) {
		if (movingPointIndex < 0) return;
		Point p = (Point)points.elementAt(movingPointIndex);
		p.setLocation(dest);
		if (poly != null) poly.compute(points);
		repaint();
	}

    private int findIndexOf(Point p, double tolerance) {
		if (p == null) return -1;
		if (points.size() <= 0) return -1;
		for (int i = 0; i < points.size(); i++) {
			Point p1 = (Point)points.elementAt(i);
			double dx = p1.x - p.x;
			double dy = p1.y - p.y;
			if (Math.sqrt(dx*dx + dy*dy) <= tolerance) return i;
		}
		return -1;
    }

    private double [] change(double [] a, int inc) {
		if (a == null || a.length <= 0) {
			if (inc > 0)
			return new double[inc];
			else return null;
		}
		if (a.length + inc <= 0) return null;
		double [] r = new double[a.length+inc];
		for (int i = 0; i < r.length && i < a.length; i++) {
			r[i] = a[i];
		}
		return r;
	}

    public void setPointsVisible(boolean flag) {
		if (flag != showPoints) {
			showPoints = flag;
			repaint();
		}
	}

	public void setPolygonType(Polygon polygon) {
		poly = polygon;
		if (poly != null)
		{
			poly.clear();
			poly.compute(points);
		}
		repaint();
	}
	
    private void addPoint(Point p) {
		points.addElement(p);
		if (poly != null)
		{
			poly.clear();
			poly.compute(points);
		}
		repaint();
	}

    public void clearPoints() {
		points.removeAllElements();
	}

    public void paint(Graphics g) {
		if (points.size() <= 0) return;
		if (showPoints) {
			for (int i = 0; i < points.size(); i++) {
				Point p = (Point)points.elementAt(i);
				g.drawRect(p.x-2, p.y-2, 5, 5);
			}
		}
		if (poly != null) poly.draw(g, points);
    }
}
