import java.awt.*;
import java.util.*;

public abstract class Spline extends Polygon {
    protected double [] x;
    protected double [] y;
    protected double [] xp;
    protected double [] yp;
    protected double [] s;

	public Spline() {
	}

	public void clear() {
		x = null;
		y = null;
		xp = null;
		yp = null;
		s = null;
	}
	
	public void draw(Graphics g, Vector p) {
		if (p.size() < 3) return;

		double t = 0;
		int m = x.length;
		int x0 = g(m, t, s, x, xp);
		int y0 = g(m, t, s, y, yp);
		int x1;
		int y1;
		while (t < s[m-1]) {
			x1 = g(m, t, s, x, xp);
			y1 = g(m, t, s, y, yp);
			g.drawLine(x0, y0, x1, y1);
			x0 = x1;
			y0 = y1;
			t += 4;
		}
		x1 = g(m, s[m-1], s, x, xp);
		y1 = g(m, s[m-1], s, y, yp);
		g.drawLine(x0, y0, x1, y1);
	}

	protected double sqr(double a) {
		return a*a;
	}

    private int g(int n, double z, double [] x, double [] y, double [] yp) {
		int i = 0;
		int j = n - 1;
		while (i+1 < j) {
			int k = (i + j) / 2;
			if (x[k] < z)
				i = k;
			else
				j = k;
		}
		double h = x[i+1] - x[i];
		double t = (z - x[i]) / h;
		double a0 = y[i];
		double a1 = y[i+1] - a0;
		double a2 = a1 - h * yp[i];
		double a3 = h * yp[i+1] - a1 - a2;
		return (int)(a0 + (a1 + (a2 + a3 * t) * (t - 1)) * t);
	}

    protected void solveTridia(int n, double [] c, double [] a, double [] b, double [] x) {
		b[n-1] = 0;
		for (int i = 0; i < n-1; i++) {
			if (c[i] != 0) {
				double t = a[i] / c[i];
				double sin = 1.0 / Math.sqrt(1 + t * t);
				double cos = t * sin;
				a[i] = a[i] * cos + c[i] * sin;
				double h = b[i];
				b[i] = h * cos + a[i+1] * sin;
				a[i+1] = -h * sin + a[i+1] * cos;
				c[i] = b[i+1] * sin;
				b[i+1] = b[i+1] * cos;
				h = x[i];
				x[i] = h * cos + x[i+1] * sin;
				x[i+1] = -h * sin + x[i+1] * cos;
			}
		}
	
		x[n-1] = x[n-1] / a[n-1];
		x[n-2] = (x[n-2] - b[n-2] * x[n-1]) / a[n-2];
		for (int i = n-3; i >= 0; i--) {
			x[i] = (x[i] - b[i] * x[i+1] - c[i] * x[i+2]) / a[i];
		}
	}

    protected void solveTridia2(int n, double [] c, double [] a, double [] b, double [] u, double [] v){
		b[n-1] = 0.0;
		for (int i = 0; i < n-1; i++) {
			if (c[i] != 0.0) {
				double t = a[i] / c[i];
				double sin = 1.0 / Math.sqrt(1 + t * t);
				double cos = t * sin;
				a[i] = a[i] * cos + c[i] * sin;
				double h = b[i];
				b[i] = h * cos + a[i+1] * sin;
				a[i+1] = -h * sin + a[i+1] * cos;
				c[i] = b[i+1] * sin;
				b[i+1] = b[i+1] * cos;
				h = u[i];
				u[i] = h * cos + u[i+1] * sin;
				u[i+1] = -h * sin + u[i+1] * cos;
				h = v[i];
				v[i] = h * cos + v[i+1] * sin;
				v[i+1] = -h * sin + v[i+1] * cos;
			}
		}
		
		u[n-1] = u[n-1] / a[n-1];
		u[n-2] = (u[n-2] - b[n-2] * u[n-1]) / a[n-2];
		v[n-1] = v[n-1] / a[n-1];
		v[n-2] = (v[n-2] - b[n-2] * v[n-1]) / a[n-2];
		for (int i = n-3; i >= 0; i--) {
			u[i] = (u[i] - b[i] * u[i+1] - c[i] * u[i+2]) / a[i];
			v[i] = (v[i] - b[i] * v[i+1] - c[i] * v[i+2]) / a[i];
		}
    }

	protected abstract void derivatives(int n, double [] x, double [] y, double [] yp);
}

